# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TliveKafkaOcSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'brokers': 'list[str]',
        'collector_settings': 'object',
        'name': 'str',
        'security': 'TlivekafkaocSchemaSecurity',
        'topics': 'list[str]'
    }

    attribute_map = {
        'brokers': 'brokers',
        'collector_settings': 'collector-settings',
        'name': 'name',
        'security': 'security',
        'topics': 'topics'
    }

    def __init__(self, brokers=None, collector_settings=None, name=None, security=None, topics=None):  # noqa: E501
        """TliveKafkaOcSchema - a model defined in Swagger"""  # noqa: E501

        self._brokers = None
        self._collector_settings = None
        self._name = None
        self._security = None
        self._topics = None
        self.discriminator = None

        self.brokers = brokers
        if collector_settings is not None:
            self.collector_settings = collector_settings
        self.name = name
        if security is not None:
            self.security = security
        if topics is not None:
            self.topics = topics

    @property
    def brokers(self):
        """Gets the brokers of this TliveKafkaOcSchema.  # noqa: E501


        :return: The brokers of this TliveKafkaOcSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._brokers

    @brokers.setter
    def brokers(self, brokers):
        """Sets the brokers of this TliveKafkaOcSchema.


        :param brokers: The brokers of this TliveKafkaOcSchema.  # noqa: E501
        :type: list[str]
        """
        if brokers is None:
            raise ValueError("Invalid value for `brokers`, must not be `None`")  # noqa: E501

        self._brokers = brokers

    @property
    def collector_settings(self):
        """Gets the collector_settings of this TliveKafkaOcSchema.  # noqa: E501


        :return: The collector_settings of this TliveKafkaOcSchema.  # noqa: E501
        :rtype: object
        """
        return self._collector_settings

    @collector_settings.setter
    def collector_settings(self, collector_settings):
        """Sets the collector_settings of this TliveKafkaOcSchema.


        :param collector_settings: The collector_settings of this TliveKafkaOcSchema.  # noqa: E501
        :type: object
        """

        self._collector_settings = collector_settings

    @property
    def name(self):
        """Gets the name of this TliveKafkaOcSchema.  # noqa: E501

        Name of this instance  # noqa: E501

        :return: The name of this TliveKafkaOcSchema.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this TliveKafkaOcSchema.

        Name of this instance  # noqa: E501

        :param name: The name of this TliveKafkaOcSchema.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        if name is not None and len(name) > 64:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `64`")  # noqa: E501
        if name is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9_-]*$', name):  # noqa: E501
            raise ValueError(r"Invalid value for `name`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._name = name

    @property
    def security(self):
        """Gets the security of this TliveKafkaOcSchema.  # noqa: E501


        :return: The security of this TliveKafkaOcSchema.  # noqa: E501
        :rtype: TlivekafkaocSchemaSecurity
        """
        return self._security

    @security.setter
    def security(self, security):
        """Sets the security of this TliveKafkaOcSchema.


        :param security: The security of this TliveKafkaOcSchema.  # noqa: E501
        :type: TlivekafkaocSchemaSecurity
        """

        self._security = security

    @property
    def topics(self):
        """Gets the topics of this TliveKafkaOcSchema.  # noqa: E501


        :return: The topics of this TliveKafkaOcSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._topics

    @topics.setter
    def topics(self, topics):
        """Sets the topics of this TliveKafkaOcSchema.


        :param topics: The topics of this TliveKafkaOcSchema.  # noqa: E501
        :type: list[str]
        """

        self._topics = topics

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TliveKafkaOcSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TliveKafkaOcSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class RuleSchemaRulepropertiesSupporteddevicesOperatingsystems(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'os_name': 'str',
        'products': 'list[RuleSchemaRulepropertiesSupporteddevicesProducts]',
        'sensors': 'list[str]'
    }

    attribute_map = {
        'os_name': 'os-name',
        'products': 'products',
        'sensors': 'sensors'
    }

    def __init__(self, os_name=None, products=None, sensors=None):  # noqa: E501
        """RuleSchemaRulepropertiesSupporteddevicesOperatingsystems - a model defined in Swagger"""  # noqa: E501

        self._os_name = None
        self._products = None
        self._sensors = None
        self.discriminator = None

        self.os_name = os_name
        if products is not None:
            self.products = products
        if sensors is not None:
            self.sensors = sensors

    @property
    def os_name(self):
        """Gets the os_name of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501

        Operating system for the supported devices  # noqa: E501

        :return: The os_name of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :rtype: str
        """
        return self._os_name

    @os_name.setter
    def os_name(self, os_name):
        """Sets the os_name of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.

        Operating system for the supported devices  # noqa: E501

        :param os_name: The os_name of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :type: str
        """
        if os_name is None:
            raise ValueError("Invalid value for `os_name`, must not be `None`")  # noqa: E501
        if os_name is not None and len(os_name) > 64:
            raise ValueError("Invalid value for `os_name`, length must be less than or equal to `64`")  # noqa: E501
        if os_name is not None and len(os_name) < 1:
            raise ValueError("Invalid value for `os_name`, length must be greater than or equal to `1`")  # noqa: E501

        self._os_name = os_name

    @property
    def products(self):
        """Gets the products of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501

        Product information of the device  # noqa: E501

        :return: The products of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :rtype: list[RuleSchemaRulepropertiesSupporteddevicesProducts]
        """
        return self._products

    @products.setter
    def products(self, products):
        """Sets the products of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.

        Product information of the device  # noqa: E501

        :param products: The products of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :type: list[RuleSchemaRulepropertiesSupporteddevicesProducts]
        """

        self._products = products

    @property
    def sensors(self):
        """Gets the sensors of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501


        :return: The sensors of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :rtype: list[str]
        """
        return self._sensors

    @sensors.setter
    def sensors(self, sensors):
        """Sets the sensors of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.


        :param sensors: The sensors of this RuleSchemaRulepropertiesSupporteddevicesOperatingsystems.  # noqa: E501
        :type: list[str]
        """

        self._sensors = sensors

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuleSchemaRulepropertiesSupporteddevicesOperatingsystems, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuleSchemaRulepropertiesSupporteddevicesOperatingsystems):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class DeviceGroupSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'authentication': 'DevicegroupSchemaAuthentication',
        'edge': 'str',
        'action_scheduler': 'DevicegroupSchemaActionscheduler',
        'description': 'str',
        'device_group_name': 'str',
        'devices': 'list[str]',
        'logging': 'DevicegroupSchemaLogging',
        'native_gpb': 'DevicegroupSchemaNativegpb',
        'flow': 'DevicegroupSchemaFlow',
        'ingest_frequency': 'list[str]',
        'raw_data': 'DevicegroupSchemaRawdata',
        'field_data': 'DevicegroupSchemaFielddata',
        'notification': 'DevicegroupSchemaNotification',
        'open_config': 'DevicegroupSchemaOpenconfig',
        'outbound_ssh': 'DevicegroupSchemaOutboundssh',
        'playbooks': 'list[str]',
        'publish': 'DevicegroupSchemaPublish',
        'reports': 'list[str]',
        'retention_policy': 'str',
        'root_cause_analysis': 'DevicegroupSchemaRootcauseanalysis',
        'scheduler': 'list[DevicegroupSchemaScheduler]',
        'variable': 'list[DevicegroupSchemaVariable]',
        'snmp': 'DevicegroupSchemaSnmp',
        'syslog': 'DevicegroupSchemaSyslog',
        'tagging_profile': 'list[str]',
        'timezone': 'str',
        'use_ingest_receive_time': 'list[object]'
    }

    attribute_map = {
        'authentication': 'authentication',
        'edge': 'edge',
        'action_scheduler': 'action-scheduler',
        'description': 'description',
        'device_group_name': 'device-group-name',
        'devices': 'devices',
        'logging': 'logging',
        'native_gpb': 'native-gpb',
        'flow': 'flow',
        'ingest_frequency': 'ingest-frequency',
        'raw_data': 'raw-data',
        'field_data': 'field-data',
        'notification': 'notification',
        'open_config': 'open-config',
        'outbound_ssh': 'outbound-ssh',
        'playbooks': 'playbooks',
        'publish': 'publish',
        'reports': 'reports',
        'retention_policy': 'retention-policy',
        'root_cause_analysis': 'root-cause-analysis',
        'scheduler': 'scheduler',
        'variable': 'variable',
        'snmp': 'snmp',
        'syslog': 'syslog',
        'tagging_profile': 'tagging-profile',
        'timezone': 'timezone',
        'use_ingest_receive_time': 'use-ingest-receive-time'
    }

    def __init__(self, authentication=None, edge=None, action_scheduler=None, description=None, device_group_name=None, devices=None, logging=None, native_gpb=None, flow=None, ingest_frequency=None, raw_data=None, field_data=None, notification=None, open_config=None, outbound_ssh=None, playbooks=None, publish=None, reports=None, retention_policy=None, root_cause_analysis=None, scheduler=None, variable=None, snmp=None, syslog=None, tagging_profile=None, timezone=None, use_ingest_receive_time=None):  # noqa: E501
        """DeviceGroupSchema - a model defined in Swagger"""  # noqa: E501

        self._authentication = None
        self._edge = None
        self._action_scheduler = None
        self._description = None
        self._device_group_name = None
        self._devices = None
        self._logging = None
        self._native_gpb = None
        self._flow = None
        self._ingest_frequency = None
        self._raw_data = None
        self._field_data = None
        self._notification = None
        self._open_config = None
        self._outbound_ssh = None
        self._playbooks = None
        self._publish = None
        self._reports = None
        self._retention_policy = None
        self._root_cause_analysis = None
        self._scheduler = None
        self._variable = None
        self._snmp = None
        self._syslog = None
        self._tagging_profile = None
        self._timezone = None
        self._use_ingest_receive_time = None
        self.discriminator = None

        if authentication is not None:
            self.authentication = authentication
        if edge is not None:
            self.edge = edge
        if action_scheduler is not None:
            self.action_scheduler = action_scheduler
        if description is not None:
            self.description = description
        self.device_group_name = device_group_name
        if devices is not None:
            self.devices = devices
        if logging is not None:
            self.logging = logging
        if native_gpb is not None:
            self.native_gpb = native_gpb
        if flow is not None:
            self.flow = flow
        if ingest_frequency is not None:
            self.ingest_frequency = ingest_frequency
        if raw_data is not None:
            self.raw_data = raw_data
        if field_data is not None:
            self.field_data = field_data
        if notification is not None:
            self.notification = notification
        if open_config is not None:
            self.open_config = open_config
        if outbound_ssh is not None:
            self.outbound_ssh = outbound_ssh
        if playbooks is not None:
            self.playbooks = playbooks
        if publish is not None:
            self.publish = publish
        if reports is not None:
            self.reports = reports
        if retention_policy is not None:
            self.retention_policy = retention_policy
        if root_cause_analysis is not None:
            self.root_cause_analysis = root_cause_analysis
        if scheduler is not None:
            self.scheduler = scheduler
        if variable is not None:
            self.variable = variable
        if snmp is not None:
            self.snmp = snmp
        if syslog is not None:
            self.syslog = syslog
        if tagging_profile is not None:
            self.tagging_profile = tagging_profile
        if timezone is not None:
            self.timezone = timezone
        if use_ingest_receive_time is not None:
            self.use_ingest_receive_time = use_ingest_receive_time

    @property
    def authentication(self):
        """Gets the authentication of this DeviceGroupSchema.  # noqa: E501


        :return: The authentication of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaAuthentication
        """
        return self._authentication

    @authentication.setter
    def authentication(self, authentication):
        """Sets the authentication of this DeviceGroupSchema.


        :param authentication: The authentication of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaAuthentication
        """

        self._authentication = authentication

    @property
    def edge(self):
        """Gets the edge of this DeviceGroupSchema.  # noqa: E501

        JFM: edge this device group belongs to. This should be of the format <organization-name>.<site-name>.<edge-name>  # noqa: E501

        :return: The edge of this DeviceGroupSchema.  # noqa: E501
        :rtype: str
        """
        return self._edge

    @edge.setter
    def edge(self, edge):
        """Sets the edge of this DeviceGroupSchema.

        JFM: edge this device group belongs to. This should be of the format <organization-name>.<site-name>.<edge-name>  # noqa: E501

        :param edge: The edge of this DeviceGroupSchema.  # noqa: E501
        :type: str
        """
        if edge is not None and len(edge) > 194:
            raise ValueError("Invalid value for `edge`, length must be less than or equal to `194`")  # noqa: E501
        if edge is not None and len(edge) < 5:
            raise ValueError("Invalid value for `edge`, length must be greater than or equal to `5`")  # noqa: E501
        if edge is not None and not re.search(r'^([a-zA-Z][a-zA-Z0-9_-]*\\.){2}[a-zA-Z][a-zA-Z0-9_-]*$', edge):  # noqa: E501
            raise ValueError(r"Invalid value for `edge`, must be a follow pattern or equal to `/^([a-zA-Z][a-zA-Z0-9_-]*\\.){2}[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._edge = edge

    @property
    def action_scheduler(self):
        """Gets the action_scheduler of this DeviceGroupSchema.  # noqa: E501


        :return: The action_scheduler of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaActionscheduler
        """
        return self._action_scheduler

    @action_scheduler.setter
    def action_scheduler(self, action_scheduler):
        """Sets the action_scheduler of this DeviceGroupSchema.


        :param action_scheduler: The action_scheduler of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaActionscheduler
        """

        self._action_scheduler = action_scheduler

    @property
    def description(self):
        """Gets the description of this DeviceGroupSchema.  # noqa: E501

        Description about the device group  # noqa: E501

        :return: The description of this DeviceGroupSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this DeviceGroupSchema.

        Description about the device group  # noqa: E501

        :param description: The description of this DeviceGroupSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def device_group_name(self):
        """Gets the device_group_name of this DeviceGroupSchema.  # noqa: E501

        Name of the group. Should be of pattern [a-zA-Z][a-zA-Z0-9-]*  # noqa: E501

        :return: The device_group_name of this DeviceGroupSchema.  # noqa: E501
        :rtype: str
        """
        return self._device_group_name

    @device_group_name.setter
    def device_group_name(self, device_group_name):
        """Sets the device_group_name of this DeviceGroupSchema.

        Name of the group. Should be of pattern [a-zA-Z][a-zA-Z0-9-]*  # noqa: E501

        :param device_group_name: The device_group_name of this DeviceGroupSchema.  # noqa: E501
        :type: str
        """
        if device_group_name is None:
            raise ValueError("Invalid value for `device_group_name`, must not be `None`")  # noqa: E501
        if device_group_name is not None and len(device_group_name) > 40:
            raise ValueError("Invalid value for `device_group_name`, length must be less than or equal to `40`")  # noqa: E501
        if device_group_name is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9-]*$', device_group_name):  # noqa: E501
            raise ValueError(r"Invalid value for `device_group_name`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9-]*$/`")  # noqa: E501

        self._device_group_name = device_group_name

    @property
    def devices(self):
        """Gets the devices of this DeviceGroupSchema.  # noqa: E501


        :return: The devices of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._devices

    @devices.setter
    def devices(self, devices):
        """Sets the devices of this DeviceGroupSchema.


        :param devices: The devices of this DeviceGroupSchema.  # noqa: E501
        :type: list[str]
        """

        self._devices = devices

    @property
    def logging(self):
        """Gets the logging of this DeviceGroupSchema.  # noqa: E501


        :return: The logging of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaLogging
        """
        return self._logging

    @logging.setter
    def logging(self, logging):
        """Sets the logging of this DeviceGroupSchema.


        :param logging: The logging of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaLogging
        """

        self._logging = logging

    @property
    def native_gpb(self):
        """Gets the native_gpb of this DeviceGroupSchema.  # noqa: E501


        :return: The native_gpb of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaNativegpb
        """
        return self._native_gpb

    @native_gpb.setter
    def native_gpb(self, native_gpb):
        """Sets the native_gpb of this DeviceGroupSchema.


        :param native_gpb: The native_gpb of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaNativegpb
        """

        self._native_gpb = native_gpb

    @property
    def flow(self):
        """Gets the flow of this DeviceGroupSchema.  # noqa: E501


        :return: The flow of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaFlow
        """
        return self._flow

    @flow.setter
    def flow(self, flow):
        """Sets the flow of this DeviceGroupSchema.


        :param flow: The flow of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaFlow
        """

        self._flow = flow

    @property
    def ingest_frequency(self):
        """Gets the ingest_frequency of this DeviceGroupSchema.  # noqa: E501


        :return: The ingest_frequency of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._ingest_frequency

    @ingest_frequency.setter
    def ingest_frequency(self, ingest_frequency):
        """Sets the ingest_frequency of this DeviceGroupSchema.


        :param ingest_frequency: The ingest_frequency of this DeviceGroupSchema.  # noqa: E501
        :type: list[str]
        """

        self._ingest_frequency = ingest_frequency

    @property
    def raw_data(self):
        """Gets the raw_data of this DeviceGroupSchema.  # noqa: E501


        :return: The raw_data of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaRawdata
        """
        return self._raw_data

    @raw_data.setter
    def raw_data(self, raw_data):
        """Sets the raw_data of this DeviceGroupSchema.


        :param raw_data: The raw_data of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaRawdata
        """

        self._raw_data = raw_data

    @property
    def field_data(self):
        """Gets the field_data of this DeviceGroupSchema.  # noqa: E501


        :return: The field_data of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaFielddata
        """
        return self._field_data

    @field_data.setter
    def field_data(self, field_data):
        """Sets the field_data of this DeviceGroupSchema.


        :param field_data: The field_data of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaFielddata
        """

        self._field_data = field_data

    @property
    def notification(self):
        """Gets the notification of this DeviceGroupSchema.  # noqa: E501


        :return: The notification of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaNotification
        """
        return self._notification

    @notification.setter
    def notification(self, notification):
        """Sets the notification of this DeviceGroupSchema.


        :param notification: The notification of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaNotification
        """

        self._notification = notification

    @property
    def open_config(self):
        """Gets the open_config of this DeviceGroupSchema.  # noqa: E501


        :return: The open_config of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaOpenconfig
        """
        return self._open_config

    @open_config.setter
    def open_config(self, open_config):
        """Sets the open_config of this DeviceGroupSchema.


        :param open_config: The open_config of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaOpenconfig
        """

        self._open_config = open_config

    @property
    def outbound_ssh(self):
        """Gets the outbound_ssh of this DeviceGroupSchema.  # noqa: E501


        :return: The outbound_ssh of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaOutboundssh
        """
        return self._outbound_ssh

    @outbound_ssh.setter
    def outbound_ssh(self, outbound_ssh):
        """Sets the outbound_ssh of this DeviceGroupSchema.


        :param outbound_ssh: The outbound_ssh of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaOutboundssh
        """

        self._outbound_ssh = outbound_ssh

    @property
    def playbooks(self):
        """Gets the playbooks of this DeviceGroupSchema.  # noqa: E501


        :return: The playbooks of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._playbooks

    @playbooks.setter
    def playbooks(self, playbooks):
        """Sets the playbooks of this DeviceGroupSchema.


        :param playbooks: The playbooks of this DeviceGroupSchema.  # noqa: E501
        :type: list[str]
        """

        self._playbooks = playbooks

    @property
    def publish(self):
        """Gets the publish of this DeviceGroupSchema.  # noqa: E501


        :return: The publish of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaPublish
        """
        return self._publish

    @publish.setter
    def publish(self, publish):
        """Sets the publish of this DeviceGroupSchema.


        :param publish: The publish of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaPublish
        """

        self._publish = publish

    @property
    def reports(self):
        """Gets the reports of this DeviceGroupSchema.  # noqa: E501


        :return: The reports of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._reports

    @reports.setter
    def reports(self, reports):
        """Sets the reports of this DeviceGroupSchema.


        :param reports: The reports of this DeviceGroupSchema.  # noqa: E501
        :type: list[str]
        """

        self._reports = reports

    @property
    def retention_policy(self):
        """Gets the retention_policy of this DeviceGroupSchema.  # noqa: E501

        Name of the retention policy to be applied  # noqa: E501

        :return: The retention_policy of this DeviceGroupSchema.  # noqa: E501
        :rtype: str
        """
        return self._retention_policy

    @retention_policy.setter
    def retention_policy(self, retention_policy):
        """Sets the retention_policy of this DeviceGroupSchema.

        Name of the retention policy to be applied  # noqa: E501

        :param retention_policy: The retention_policy of this DeviceGroupSchema.  # noqa: E501
        :type: str
        """
        if retention_policy is not None and len(retention_policy) > 64:
            raise ValueError("Invalid value for `retention_policy`, length must be less than or equal to `64`")  # noqa: E501
        if retention_policy is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9_-]*$', retention_policy):  # noqa: E501
            raise ValueError(r"Invalid value for `retention_policy`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._retention_policy = retention_policy

    @property
    def root_cause_analysis(self):
        """Gets the root_cause_analysis of this DeviceGroupSchema.  # noqa: E501


        :return: The root_cause_analysis of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaRootcauseanalysis
        """
        return self._root_cause_analysis

    @root_cause_analysis.setter
    def root_cause_analysis(self, root_cause_analysis):
        """Sets the root_cause_analysis of this DeviceGroupSchema.


        :param root_cause_analysis: The root_cause_analysis of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaRootcauseanalysis
        """

        self._root_cause_analysis = root_cause_analysis

    @property
    def scheduler(self):
        """Gets the scheduler of this DeviceGroupSchema.  # noqa: E501

        List of schedulers associated with the playbook instances  # noqa: E501

        :return: The scheduler of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[DevicegroupSchemaScheduler]
        """
        return self._scheduler

    @scheduler.setter
    def scheduler(self, scheduler):
        """Sets the scheduler of this DeviceGroupSchema.

        List of schedulers associated with the playbook instances  # noqa: E501

        :param scheduler: The scheduler of this DeviceGroupSchema.  # noqa: E501
        :type: list[DevicegroupSchemaScheduler]
        """

        self._scheduler = scheduler

    @property
    def variable(self):
        """Gets the variable of this DeviceGroupSchema.  # noqa: E501

        Playbook variable configuration  # noqa: E501

        :return: The variable of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[DevicegroupSchemaVariable]
        """
        return self._variable

    @variable.setter
    def variable(self, variable):
        """Sets the variable of this DeviceGroupSchema.

        Playbook variable configuration  # noqa: E501

        :param variable: The variable of this DeviceGroupSchema.  # noqa: E501
        :type: list[DevicegroupSchemaVariable]
        """

        self._variable = variable

    @property
    def snmp(self):
        """Gets the snmp of this DeviceGroupSchema.  # noqa: E501


        :return: The snmp of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaSnmp
        """
        return self._snmp

    @snmp.setter
    def snmp(self, snmp):
        """Sets the snmp of this DeviceGroupSchema.


        :param snmp: The snmp of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaSnmp
        """

        self._snmp = snmp

    @property
    def syslog(self):
        """Gets the syslog of this DeviceGroupSchema.  # noqa: E501


        :return: The syslog of this DeviceGroupSchema.  # noqa: E501
        :rtype: DevicegroupSchemaSyslog
        """
        return self._syslog

    @syslog.setter
    def syslog(self, syslog):
        """Sets the syslog of this DeviceGroupSchema.


        :param syslog: The syslog of this DeviceGroupSchema.  # noqa: E501
        :type: DevicegroupSchemaSyslog
        """

        self._syslog = syslog

    @property
    def tagging_profile(self):
        """Gets the tagging_profile of this DeviceGroupSchema.  # noqa: E501


        :return: The tagging_profile of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._tagging_profile

    @tagging_profile.setter
    def tagging_profile(self, tagging_profile):
        """Sets the tagging_profile of this DeviceGroupSchema.


        :param tagging_profile: The tagging_profile of this DeviceGroupSchema.  # noqa: E501
        :type: list[str]
        """

        self._tagging_profile = tagging_profile

    @property
    def timezone(self):
        """Gets the timezone of this DeviceGroupSchema.  # noqa: E501

        Timezone in the format +/-hh:mm, Example: -08:00  # noqa: E501

        :return: The timezone of this DeviceGroupSchema.  # noqa: E501
        :rtype: str
        """
        return self._timezone

    @timezone.setter
    def timezone(self, timezone):
        """Sets the timezone of this DeviceGroupSchema.

        Timezone in the format +/-hh:mm, Example: -08:00  # noqa: E501

        :param timezone: The timezone of this DeviceGroupSchema.  # noqa: E501
        :type: str
        """
        if timezone is not None and not re.search(r'^((\\+|-)((([0-1][0-9])|(2[0-3])):([0-5][0-9])))$', timezone):  # noqa: E501
            raise ValueError(r"Invalid value for `timezone`, must be a follow pattern or equal to `/^((\\+|-)((([0-1][0-9])|(2[0-3])):([0-5][0-9])))$/`")  # noqa: E501

        self._timezone = timezone

    @property
    def use_ingest_receive_time(self):
        """Gets the use_ingest_receive_time of this DeviceGroupSchema.  # noqa: E501

        Enable using ingest receive time in formulas like elapsed-time and rate-of-change  # noqa: E501

        :return: The use_ingest_receive_time of this DeviceGroupSchema.  # noqa: E501
        :rtype: list[object]
        """
        return self._use_ingest_receive_time

    @use_ingest_receive_time.setter
    def use_ingest_receive_time(self, use_ingest_receive_time):
        """Sets the use_ingest_receive_time of this DeviceGroupSchema.

        Enable using ingest receive time in formulas like elapsed-time and rate-of-change  # noqa: E501

        :param use_ingest_receive_time: The use_ingest_receive_time of this DeviceGroupSchema.  # noqa: E501
        :type: list[object]
        """

        self._use_ingest_receive_time = use_ingest_receive_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DeviceGroupSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DeviceGroupSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class RuleSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'field': 'list[RuleSchemaField]',
        'function': 'list[RuleSchemaFunction]',
        'keys': 'list[str]',
        'network_rule': 'list[object]',
        'disable_no_data_alarm': 'list[object]',
        'rule_frequency': 'str',
        'rule_name': 'str',
        'sensor': 'list[RuleSchemaSensor1]',
        'synopsis': 'str',
        'field_aggregation_time_range': 'str',
        'trigger': 'list[RuleSchemaTrigger]',
        'variable': 'list[RuleSchemaVariable]',
        'vector': 'list[RuleSchemaVector]',
        'rule_properties': 'RuleSchemaRuleproperties'
    }

    attribute_map = {
        'description': 'description',
        'field': 'field',
        'function': 'function',
        'keys': 'keys',
        'network_rule': 'network-rule',
        'disable_no_data_alarm': 'disable-no-data-alarm',
        'rule_frequency': 'rule-frequency',
        'rule_name': 'rule-name',
        'sensor': 'sensor',
        'synopsis': 'synopsis',
        'field_aggregation_time_range': 'field-aggregation-time-range',
        'trigger': 'trigger',
        'variable': 'variable',
        'vector': 'vector',
        'rule_properties': 'rule-properties'
    }

    def __init__(self, description=None, field=None, function=None, keys=None, network_rule=None, disable_no_data_alarm=None, rule_frequency=None, rule_name=None, sensor=None, synopsis=None, field_aggregation_time_range=None, trigger=None, variable=None, vector=None, rule_properties=None):  # noqa: E501
        """RuleSchema - a model defined in Swagger"""  # noqa: E501

        self._description = None
        self._field = None
        self._function = None
        self._keys = None
        self._network_rule = None
        self._disable_no_data_alarm = None
        self._rule_frequency = None
        self._rule_name = None
        self._sensor = None
        self._synopsis = None
        self._field_aggregation_time_range = None
        self._trigger = None
        self._variable = None
        self._vector = None
        self._rule_properties = None
        self.discriminator = None

        if description is not None:
            self.description = description
        if field is not None:
            self.field = field
        if function is not None:
            self.function = function
        if keys is not None:
            self.keys = keys
        if network_rule is not None:
            self.network_rule = network_rule
        if disable_no_data_alarm is not None:
            self.disable_no_data_alarm = disable_no_data_alarm
        if rule_frequency is not None:
            self.rule_frequency = rule_frequency
        self.rule_name = rule_name
        if sensor is not None:
            self.sensor = sensor
        if synopsis is not None:
            self.synopsis = synopsis
        if field_aggregation_time_range is not None:
            self.field_aggregation_time_range = field_aggregation_time_range
        if trigger is not None:
            self.trigger = trigger
        if variable is not None:
            self.variable = variable
        if vector is not None:
            self.vector = vector
        if rule_properties is not None:
            self.rule_properties = rule_properties

    @property
    def description(self):
        """Gets the description of this RuleSchema.  # noqa: E501

        Description about the rule  # noqa: E501

        :return: The description of this RuleSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this RuleSchema.

        Description about the rule  # noqa: E501

        :param description: The description of this RuleSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def field(self):
        """Gets the field of this RuleSchema.  # noqa: E501


        :return: The field of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaField]
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this RuleSchema.


        :param field: The field of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaField]
        """

        self._field = field

    @property
    def function(self):
        """Gets the function of this RuleSchema.  # noqa: E501


        :return: The function of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaFunction]
        """
        return self._function

    @function.setter
    def function(self, function):
        """Sets the function of this RuleSchema.


        :param function: The function of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaFunction]
        """

        self._function = function

    @property
    def keys(self):
        """Gets the keys of this RuleSchema.  # noqa: E501


        :return: The keys of this RuleSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._keys

    @keys.setter
    def keys(self, keys):
        """Sets the keys of this RuleSchema.


        :param keys: The keys of this RuleSchema.  # noqa: E501
        :type: list[str]
        """

        self._keys = keys

    @property
    def network_rule(self):
        """Gets the network_rule of this RuleSchema.  # noqa: E501

        Flag to denote a network rule  # noqa: E501

        :return: The network_rule of this RuleSchema.  # noqa: E501
        :rtype: list[object]
        """
        return self._network_rule

    @network_rule.setter
    def network_rule(self, network_rule):
        """Sets the network_rule of this RuleSchema.

        Flag to denote a network rule  # noqa: E501

        :param network_rule: The network_rule of this RuleSchema.  # noqa: E501
        :type: list[object]
        """

        self._network_rule = network_rule

    @property
    def disable_no_data_alarm(self):
        """Gets the disable_no_data_alarm of this RuleSchema.  # noqa: E501

        Disable No Data Alarm  # noqa: E501

        :return: The disable_no_data_alarm of this RuleSchema.  # noqa: E501
        :rtype: list[object]
        """
        return self._disable_no_data_alarm

    @disable_no_data_alarm.setter
    def disable_no_data_alarm(self, disable_no_data_alarm):
        """Sets the disable_no_data_alarm of this RuleSchema.

        Disable No Data Alarm  # noqa: E501

        :param disable_no_data_alarm: The disable_no_data_alarm of this RuleSchema.  # noqa: E501
        :type: list[object]
        """

        self._disable_no_data_alarm = disable_no_data_alarm

    @property
    def rule_frequency(self):
        """Gets the rule_frequency of this RuleSchema.  # noqa: E501

        Frequency at which the rule’s field, reference, and vector elements should be computed. Required only when a rule doesn’t have a sensor defined. Specify integer >= 0 followed by s/m/h/d/w/y representing seconds/minutes/hours/days/weeks/years. Eg: 2s  # noqa: E501

        :return: The rule_frequency of this RuleSchema.  # noqa: E501
        :rtype: str
        """
        return self._rule_frequency

    @rule_frequency.setter
    def rule_frequency(self, rule_frequency):
        """Sets the rule_frequency of this RuleSchema.

        Frequency at which the rule’s field, reference, and vector elements should be computed. Required only when a rule doesn’t have a sensor defined. Specify integer >= 0 followed by s/m/h/d/w/y representing seconds/minutes/hours/days/weeks/years. Eg: 2s  # noqa: E501

        :param rule_frequency: The rule_frequency of this RuleSchema.  # noqa: E501
        :type: str
        """
        if rule_frequency is not None and not re.search(r'^[1-9][0-9]*[smhdwy]$', rule_frequency):  # noqa: E501
            raise ValueError(r"Invalid value for `rule_frequency`, must be a follow pattern or equal to `/^[1-9][0-9]*[smhdwy]$/`")  # noqa: E501

        self._rule_frequency = rule_frequency

    @property
    def rule_name(self):
        """Gets the rule_name of this RuleSchema.  # noqa: E501

        Name of the rule. Should be of pattern [a-z][a-z0-9_-]*  # noqa: E501

        :return: The rule_name of this RuleSchema.  # noqa: E501
        :rtype: str
        """
        return self._rule_name

    @rule_name.setter
    def rule_name(self, rule_name):
        """Sets the rule_name of this RuleSchema.

        Name of the rule. Should be of pattern [a-z][a-z0-9_-]*  # noqa: E501

        :param rule_name: The rule_name of this RuleSchema.  # noqa: E501
        :type: str
        """
        if rule_name is None:
            raise ValueError("Invalid value for `rule_name`, must not be `None`")  # noqa: E501
        if rule_name is not None and len(rule_name) > 64:
            raise ValueError("Invalid value for `rule_name`, length must be less than or equal to `64`")  # noqa: E501
        if rule_name is not None and not re.search(r'^[a-z][a-z0-9_-]*$', rule_name):  # noqa: E501
            raise ValueError(r"Invalid value for `rule_name`, must be a follow pattern or equal to `/^[a-z][a-z0-9_-]*$/`")  # noqa: E501

        self._rule_name = rule_name

    @property
    def sensor(self):
        """Gets the sensor of this RuleSchema.  # noqa: E501


        :return: The sensor of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaSensor1]
        """
        return self._sensor

    @sensor.setter
    def sensor(self, sensor):
        """Sets the sensor of this RuleSchema.


        :param sensor: The sensor of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaSensor1]
        """

        self._sensor = sensor

    @property
    def synopsis(self):
        """Gets the synopsis of this RuleSchema.  # noqa: E501

        Synopsis about the rule  # noqa: E501

        :return: The synopsis of this RuleSchema.  # noqa: E501
        :rtype: str
        """
        return self._synopsis

    @synopsis.setter
    def synopsis(self, synopsis):
        """Sets the synopsis of this RuleSchema.

        Synopsis about the rule  # noqa: E501

        :param synopsis: The synopsis of this RuleSchema.  # noqa: E501
        :type: str
        """

        self._synopsis = synopsis

    @property
    def field_aggregation_time_range(self):
        """Gets the field_aggregation_time_range of this RuleSchema.  # noqa: E501

        How much back in time should we look for field aggregation. Specify positive integer followed by o/s/m/h/d/w/y/offset representing seconds/minutes/hours/days/weeks/years. Eg: 2s  # noqa: E501

        :return: The field_aggregation_time_range of this RuleSchema.  # noqa: E501
        :rtype: str
        """
        return self._field_aggregation_time_range

    @field_aggregation_time_range.setter
    def field_aggregation_time_range(self, field_aggregation_time_range):
        """Sets the field_aggregation_time_range of this RuleSchema.

        How much back in time should we look for field aggregation. Specify positive integer followed by o/s/m/h/d/w/y/offset representing seconds/minutes/hours/days/weeks/years. Eg: 2s  # noqa: E501

        :param field_aggregation_time_range: The field_aggregation_time_range of this RuleSchema.  # noqa: E501
        :type: str
        """
        if field_aggregation_time_range is not None and not re.search(r'^[1-9][0-9]*(o|s|m|h|d|w|y|offset)$', field_aggregation_time_range):  # noqa: E501
            raise ValueError(r"Invalid value for `field_aggregation_time_range`, must be a follow pattern or equal to `/^[1-9][0-9]*(o|s|m|h|d|w|y|offset)$/`")  # noqa: E501

        self._field_aggregation_time_range = field_aggregation_time_range

    @property
    def trigger(self):
        """Gets the trigger of this RuleSchema.  # noqa: E501


        :return: The trigger of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaTrigger]
        """
        return self._trigger

    @trigger.setter
    def trigger(self, trigger):
        """Sets the trigger of this RuleSchema.


        :param trigger: The trigger of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaTrigger]
        """

        self._trigger = trigger

    @property
    def variable(self):
        """Gets the variable of this RuleSchema.  # noqa: E501

        Playbook variable configuration  # noqa: E501

        :return: The variable of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaVariable]
        """
        return self._variable

    @variable.setter
    def variable(self, variable):
        """Sets the variable of this RuleSchema.

        Playbook variable configuration  # noqa: E501

        :param variable: The variable of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaVariable]
        """

        self._variable = variable

    @property
    def vector(self):
        """Gets the vector of this RuleSchema.  # noqa: E501


        :return: The vector of this RuleSchema.  # noqa: E501
        :rtype: list[RuleSchemaVector]
        """
        return self._vector

    @vector.setter
    def vector(self, vector):
        """Sets the vector of this RuleSchema.


        :param vector: The vector of this RuleSchema.  # noqa: E501
        :type: list[RuleSchemaVector]
        """

        self._vector = vector

    @property
    def rule_properties(self):
        """Gets the rule_properties of this RuleSchema.  # noqa: E501


        :return: The rule_properties of this RuleSchema.  # noqa: E501
        :rtype: RuleSchemaRuleproperties
        """
        return self._rule_properties

    @rule_properties.setter
    def rule_properties(self, rule_properties):
        """Sets the rule_properties of this RuleSchema.


        :param rule_properties: The rule_properties of this RuleSchema.  # noqa: E501
        :type: RuleSchemaRuleproperties
        """

        self._rule_properties = rule_properties

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuleSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuleSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from jnpr.healthbot.swagger.models import *
from jnpr.healthbot.exception import SchemaError, NotFoundError
from jnpr.healthbot.modules.playbooks import PlayBookInstanceBuilder

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def _addMethod(verb, dict):
    """Make a get or add method and add it to dict."""
    if verb == 'add':
        fields = dict.pop('_add')
        dict[verb] = _makeAdd(fields)
    if verb == 'get':
        fields = dict.pop('_get')
        dict[verb] = _makeGet(fields)


def _makeAdd(fields):
    """Return a method that can be used to add Schema."""
    schemaClass = fields.get("schema")
    logger = fields.get("logger")
    urlfor = fields.get("url_for")
    payload_key = fields.get("payload_key")

    def add(self, schema, **kwargs):
        if schema is None:
            schema = schemaClass(**kwargs)
        else:
            if not isinstance(schema, schemaClass):
                raise SchemaError(schemaClass)
        payload = self.hbot._create_payload(schema)
        url = self.hbot.urlfor.__getattribute__(urlfor)(
            payload[payload_key])
        response = self.api.post(url, json=payload)
        if response.status_code != 200:
            logger.error(response.text)
        response.raise_for_status()
        return True
    return add


def _makeGet(fields):
    """Return a method that can be used to add Schema."""
    schemaClass = fields.get("schema")
    logger = fields.get("logger")
    urlfor = fields.get("url_for")
    payload_key = fields.get("payload_key")
    uncommitted = fields.get("uncommitted", True)
    return_response_json = fields.get("return_response_json", False)

    def get(self, name: str = None, uncommitted: bool = uncommitted):
        if name is not None:
            url = self.hbot.urlfor.__getattribute__(urlfor)(
                name)
            if uncommitted:
                url += self.hbot.apiopt_candidate
            response = self.api.get(url)
            if response.status_code != 200:
                logger.error(response.text)
            response.raise_for_status()
            return self.hbot._create_schema(response, schemaClass)
        else:
            url = self.hbot.urlfor.__getattribute__(urlfor)()
            if uncommitted:
                url += self.hbot.apiopt_candidate
            response = self.api.get(url)
            if response.status_code == 404:
                return []
            response_json = response.json()
            if response.status_code != 200:
                logger.error(response.text)
                raise NotFoundError(response_json)
            response.raise_for_status()
            return_list = []
            if return_response_json:
                return response_json
            items = response_json[payload_key]
            for item in items:
                obj = self.hbot._create_schema(
                    item, schemaClass)
                return_list.append(obj)
            return return_list
    return get


class AutoGenClass(type):
    """Adds add/get methods to the class."""

    def __new__(cls, name, bases, dict):
        if '_add' in dict:
            _addMethod('add', dict)
        if '_get' in dict:
            _addMethod('get', dict)
        return type.__new__(cls, name, bases, dict)

from jnpr.healthbot.healthbot import HealthBotClient



from jnpr.healthbot.swagger.models.datastore_schema import DatastoreSchema
from jnpr.healthbot.swagger.models.panel_schema import PanelSchema
from jnpr.healthbot.swagger.models.panel_data_schema import PanelDataSchema
from jnpr.healthbot.swagger.models.hb_graphs import HbGraphs
from jnpr.healthbot.swagger.models.hb_graphs_query import HbGraphsQuery
from jnpr.healthbot.swagger.models.panel_data_schema_add_group_by_tag \
    import PanelDataSchemaAddGroupByTag
from jnpr.healthbot.swagger.models.panel_data_schema_add_where_condition \
    import PanelDataSchemaAddWhereCondition
from jnpr.healthbot.swagger.models.panel_data_schema_add_where_condition_key_or_field \
    import PanelDataSchemaAddWhereConditionKeyOrField
from jnpr.healthbot.swagger.models.panel_data_schema_add_where_condition_key_or_field_value \
    import PanelDataSchemaAddWhereConditionKeyOrFieldValue
from jnpr.healthbot.swagger.models.panel_data_schema_add_where_condition_operator \
    import PanelDataSchemaAddWhereConditionOperator
from jnpr.healthbot.swagger.models.panel_data_schema_group_by_fill \
    import PanelDataSchemaGroupByFill
from jnpr.healthbot.swagger.models.panel_data_schema_group_by_time \
    import PanelDataSchemaGroupByTime
from jnpr.healthbot.swagger.models.panel_data_schema_selected_aggregation \
    import PanelDataSchemaSelectedAggregation
from jnpr.healthbot.swagger.models.panel_data_schema_selected_field \
    import PanelDataSchemaSelectedField
from jnpr.healthbot.swagger.models.panel_data_schema_selected_group \
    import PanelDataSchemaSelectedGroup
from jnpr.healthbot.swagger.models.panel_data_schema_selected_topic \
    import PanelDataSchemaSelectedTopic
from jnpr.healthbot.swagger.models.panel_data_schema_selected_transformation \
    import PanelDataSchemaSelectedTransformation
from jnpr.healthbot.swagger.models.panel_data_schema_selected_device \
    import PanelDataSchemaSelectedDevice
from jnpr.healthbot.swagger.models.panel_schema_panel_type \
    import PanelSchemaPanelType
from jnpr.healthbot.swagger.models.panel_schema_time_range import PanelSchemaTimeRange
from jnpr.healthbot.swagger.models.panel_schema_unit_type import PanelSchemaUnitType
from jnpr.healthbot.exception import NotFoundError
from jnpr.healthbot.modules import BaseModule
from uuid import uuid4
import logging

logger = logging.getLogger(__file__)


class HBCharts(BaseModule):

    def __init__(self, hbot):
        """
        :param object hbot: :class:`jnpr.healthbot.HealthBotClient` client instance
        """
        super().__init__(hbot)

    def get_add_canvas_schema(self, canvas_name, graphs):
        # TODO Add comments

        def _get_database_name(group_name, group_type="device", device_name=None):
            if group_type == "network":
                return f'hb-{self.hbot.tenant}:__network:{group_name}'
            else:
                return f'hb-{self.hbot.tenant}:{group_name}:{device_name}'

        def _get_panel_uid():
            return str(uuid4().hex)[0:20]

        def _get_panel_query(panel_data_list):
            queries = []
            for panel in panel_data_list:
                logical_operator = panel.selected_logical_operator
                time = panel.group_by_time.value
                fill = panel.group_by_fill.value
                key = panel.add_group_by_tag
                time_filter = "$timeFilter"
                db_name = _get_database_name(group_type=panel.group_type,
                                             group_name=panel.selected_group.value,
                                             device_name=panel.selected_device.value)
                retention_policy = panel.selected_topic.retention_policy

                # Select expression
                select_expr = f'"{panel.selected_field.value}"'
                if panel.selected_aggregation.value is not None:
                    select_expr = panel.selected_aggregation.value + f"({select_expr})"
                if panel.selected_transformation != "":
                    select_expr = panel.selected_transformation.value + f"({select_expr})"

                # From expression
                from_expr = f'\"{db_name}"."{retention_policy}"."{panel.selected_topic.value}"'

                # Where expression
                where_expr = f''
                where_count = len(panel.add_where_condition)
                for count in range(0, where_count):
                    key_or_field = panel.add_where_condition[count].key_or_field
                    operator = panel.add_where_condition[count].operator
                    key_or_field_value = panel.add_where_condition[count].key_or_field_value
                    opr_value = operator.value
                    logic_operator = ""
                    if count != (where_count - 1):
                        logic_operator = logical_operator[count - 0]
                    if (opr_value == "=~") or (opr_value == "!~"):
                        where_expr = f'{where_expr} {logic_operator}'
                        where_expr = f'{where_expr} "{key_or_field.value}" {opr_value} /{key_or_field_value.value} / '
                    elif key_or_field.data_type == "number":
                        where_expr = f'{where_expr} {logic_operator}'
                        where_expr = f'{where_expr} "{key_or_field.value}" {opr_value} {key_or_field_value.value}'
                    else:
                        where_expr = f'{where_expr} {logic_operator}'
                        where_expr = f'''{where_expr} "{key_or_field.value}" {opr_value} '{key_or_field_value.value}' '''

                    if where_expr != "":
                        where_expr = f'{time_filter} AND ({where_expr.rstrip().lstrip()})'
                    else:
                        where_expr = f'{time_filter}'

                # Group by Expression
                group_by_expr = f'time({time}) fill({fill})'
                if len(key) > 0:
                    group_by_expr = ""
                    for k in key:
                        group_by_expr = group_by_expr + f', "{k.value}"'
                    group_by_expr = f'time({time}) {group_by_expr} fill({fill})'

                query_string = f"SELECT {select_expr} FROM {from_expr} WHERE {where_expr} GROUP BY {group_by_expr}"
                queries.append(query_string)

            return queries

        def _get_panel_data(graph):
            group_by_fill_mapping = {
                "fill(null)": 'null',
                "none": 'none'
            }
            panel_data_list = []
            for query in graph.query:
                add_group_by_tag = []
                group_by_keys = query.group_by_tag_key
                for key in group_by_keys:
                    add_group_by_tag.append(
                        PanelDataSchemaAddGroupByTag(data_type='string', label=key, type="key", value=key))

                add_where_condition = []
                selected_logical_operator = []
                for condition in query.where:
                    # Validate if key or field and exists
                    if condition.get('key') in query.key_dict.keys():
                        type = 'key'
                        datatype = 'string'
                        label = condition.get('key')
                        value = condition.get('value')
                        # Validate value of tags #TODO
                    else:
                        field_exists = False
                        for field in query.field_list:
                            if field['name'] == condition.get('key'):
                                current_field = field
                                field_exists = True
                                break
                        if not field_exists:
                            raise ValueError("The field {0} does not exists for the measurement "
                                             "{1}, please configure valid measurement"
                                             .format(query.field_name, query.measurement_name))
                        type = 'field'
                        datatype = current_field['type']
                        label = condition.get('key')
                        value = condition.get('value')

                    add_where_condition.append(PanelDataSchemaAddWhereCondition(
                        key_or_field=PanelDataSchemaAddWhereConditionKeyOrField(data_type=datatype, label=label,
                                                                                type=type, value=label),
                        key_or_field_value=PanelDataSchemaAddWhereConditionKeyOrFieldValue(label=value, value=value),
                        operator=PanelDataSchemaAddWhereConditionOperator(label=condition.get("operator"),
                                                                          value=condition.get("operator"))
                    ))
                    logical_operator = condition.get('logical_operation', None)
                    if logical_operator is not None:
                        selected_logical_operator.append(logical_operator)

                transformation = ""
                if query.transformation is not None:
                    transformation = PanelDataSchemaSelectedTransformation(label=query.transformation,
                                                                           value=query.transformation)

                panel_data = PanelDataSchema(
                    add_group_by_tag=add_group_by_tag,
                    add_where_condition=add_where_condition,
                    change_where_key_field=[],
                    group_by_fill=PanelDataSchemaGroupByFill(label=query.group_by_fill,
                                                             value=group_by_fill_mapping[query.group_by_fill]),
                    group_by_time=PanelDataSchemaGroupByTime(label=query.group_by_interval,
                                                             value=query.group_by_interval),
                    group_type=query.group_type,
                    selected_aggregation=PanelDataSchemaSelectedAggregation(label=str(query.field_aggregation) + "()",
                                                                            value=query.field_aggregation),
                    selected_device=PanelDataSchemaSelectedDevice(label=query.device_name, value=query.device_name),
                    selected_dropdown=[],
                    selected_field=PanelDataSchemaSelectedField(label=query.field_name, value=query.field_name),
                    selected_group=PanelDataSchemaSelectedGroup(label=query.group_name, type=query.group_type,
                                                                value=query.group_name),
                    selected_logical_operator=selected_logical_operator,
                    selected_topic=PanelDataSchemaSelectedTopic(label=query.measurement_name,
                                                                retention_policy=query.retention_policy,
                                                                type=query.measurement_type,
                                                                value=query.measurement_name),
                    selected_transformation=transformation
                )
                panel_data_list.append(panel_data)
            return panel_data_list

        def _get_snake_case(string):
            temp = string.split('_')
            return temp[0] + ''.join(ele.title() for ele in temp[1:])

        def _get_json(json_data):
            keys = list(json_data.keys())
            for key in keys:
                if isinstance(json_data[key], dict):
                    json_data[key] = _get_json(json_data[key])
                elif isinstance(json_data[key], list):
                    for i in range(0, len(json_data[key])):
                        if isinstance(json_data[key][i], dict):
                            json_data[key][i] = _get_json(json_data[key][i])
                json_data[_get_snake_case(key)] = json_data.pop(key)

            return json_data

        def _validate_data(canvas_name, graphs):
            """
            Validating if the following compulsory fields are passed by user:
            Compulsory fields: canvas_name, graph_name, graph_type, time_range,
            group_name, device_name, measurement_name, field_name

            If the required arguments are not passed return False with the error message
            """
            # Sample submission link
            sample_submission = " For more details, please refer the sample submission here {link}"\
                .format(link="")
            if (canvas_name is None) or (canvas_name == ""):
                return False, "Please enter a valid canvas name."

            for graph in graphs:
                if (graph.graph_name is None) or (graph.graph_name == ""):
                    return False, "Please enter a valid graph name"

                if not isinstance(graph, HbGraphs):
                    return False, "Graphs is not of type HbGraphs Schema." + sample_submission

                if graph.time_range is None:
                    return False, " `time_range` parameter missing in HbGraphs object." + sample_submission
                if graph.graph_type is None:
                    return False, " `graph_type` parameter missing in HbGraphs object." + sample_submission

                if (graph.query is None) or (graph.query is []):
                    return False, "`query` parameter missing in HbGraphs object." + sample_submission

                for query in graph.query:
                    if not isinstance(query, HbGraphsQuery):
                        return False, "Query value(s) are not of type HbGraphsQuery Schema." + sample_submission

                    if query.group_name is None:
                        return False, " `group_name` parameter missing in HbGraphs object." + sample_submission

                    if query.measurement_name is None:
                        return False, " `measurement_name` parameter missing in HbGraphs object." + sample_submission
                    if query.field_name is None:
                        return False, " `field_name` parameter missing in HbGraphs object." + sample_submission
                    if query.where is not None:
                        if not isinstance(query.where, list):
                            return False, "Query's where parameters value(s) must be a list." + sample_submission

                        for where_condition in query.where:
                            keys = ["key", "value", "operator"]
                            for key in keys:
                                if key not in where_condition.keys():
                                    return False, 'Invalid input for where condition. Sample: where=' \
                                                  '[{"key":"abc","operator":"==","value":"gbha", ' \
                                                  '"logical_operation":"and"}.' + sample_submission

                            valid_operators = ["=", "!=", "=~", "!="]
                            if where_condition['operator'] not in valid_operators:
                                return False, 'Invalid operator value for in where parameter of Query, ' \
                                              'valid operators: "=", "!=", "=~", "!=" .' + sample_submission
                        if len(query.where) > 1:
                            for where_condition in query.where[:-1]:
                                if 'logical_operation' not in where_condition.keys():
                                    return False, 'Invalid input for where condition. Sample: where=' \
                                                  '[{"key":"abc","operator":"==","value":"gbha", ' \
                                                  '"logical_operation":"and"}.' + sample_submission
                                valid_logical_operation_keys = ['AND', 'OR']
                                if where_condition['logical_operation'] not in valid_logical_operation_keys:
                                    return False, 'Invalid logical_operation value for in where parameter ' \
                                                  'of Query, valid operators: "AND", "OR" .' + sample_submission

            return True, "Validation successful!"

        def _get_response(url):
            response = self.api.get(url)

            if response.status_code == 404:
                return []

            response_json = response.json()
            if response.status_code != 200:
                logger.error(response.text)
                raise NotFoundError(response_json)
            return response_json

        def _set_default_values(graphs):
            dg_list = _get_response(self.hbot.urlfor.device_group_list())
            ng_list = _get_response(self.hbot.urlfor.network_group_list())
            for graph in graphs:
                for query in graph.query:
                    if query.where is None:
                        query.where = []
                    query.field_list = []
                    query.key_dict = {}
                    if query.group_type is None:
                        if query.group_name in dg_list:
                            query.group_type = "device"
                        elif query.group_name in ng_list:
                            query.group_type = "network"
                        else:
                            raise ValueError("The group : {} does not exist, please configure a "
                                             "valid device/network group".format(query.group_name))

                    if query.group_type == "device":
                        if query.group_name not in dg_list:
                            raise ValueError("The device group : {} does not exist".format(query.group_name))

                        if query.device_name is None:
                            raise ValueError("No device configured for graph {}"
                                             .format(graph.graph_name))
                        else:
                            device_list = _get_response(self.hbot.urlfor.device())
                            if query.device_name not in device_list:
                                raise ValueError("Device {} does not exist, please configure "
                                                 "valid device".format(query.device_name))
                            measurements_details = _get_response(self.hbot.urlfor.dg_measurements_list
                                                                 (device_id=query.device_name,
                                                                  device_group=query.group_name))
                            measurement_exists = False
                            for measurement in measurements_details:
                                if measurement['name'] == query.measurement_name:
                                    current_measurement = measurement
                                    measurement_exists = True
                                    break
                            if not measurement_exists:
                                raise ValueError("The measurement {0} not present for the device group {1} and "
                                                 "device {2}, please configure a valid measurement".format
                                                 (query.measurement_name, query.group_name, query.device_name))
                            query.measurement_type = current_measurement['type']
                            query.retention_policy = current_measurement['retention_policy']
                            fields_list = _get_response(self.hbot.urlfor.dg_measurement_fields(
                                device_id=query.device_name, device_group=query.group_name,
                                table_name=query.measurement_name))
                            query.field_list = fields_list
                            field_exists = False
                            for field in fields_list:
                                if field['name'] == query.field_name:
                                    current_field = field
                                    field_exists = True
                                    break
                            if not field_exists:
                                raise ValueError("The field {0} does not exists for the measurement "
                                                 "{1}, please configure valid measurement"
                                                 .format(query.field_name, query.measurement_name))
                            query.field_type = current_field['type']

                            tags_dict = _get_response(self.hbot.urlfor.dg_measurement_tags(
                                device_id=query.device_name, device_group=query.group_name,
                                table_name=query.measurement_name))
                            query.key_dict = tags_dict

                    elif query.group_type == "network":
                        if query.group_name not in ng_list:
                            raise ValueError("The network group : {} does not exist".format(query.group_name))
                        measurements_details = _get_response(self.hbot.urlfor.ng_measurements_list
                                                             (network_group=query.group_name))
                        measurement_exists = False
                        for measurement in measurements_details:
                            if measurement['name'] == query.measurement_name:
                                current_measurement = measurement
                                measurement_exists = True
                                break
                        if not measurement_exists:
                            raise ValueError("The measurement {0} not present for the network group {1} "
                                             ", please configure a valid measurement".format
                                             (query.measurement_name, query.group_name))
                        query.measurement_type = current_measurement['type']
                        query.retention_policy = current_measurement['retention_policy']
                        fields_list = _get_response(self.hbot.urlfor.ng_measurement_fields(
                            network_group=query.group_name, table_name=query.measurement_name))
                        query.fields_list = query.fields_list + fields_list
                        field_exists = False
                        for field in fields_list:
                            if field['name'] == query.field_name:
                                current_field = field
                                field_exists = True
                                break
                        if not field_exists:
                            raise ValueError("The field {0} does not exists for the measurement "
                                             "{1}, please configure valid measurement"
                                             .format(query.field_name, query.measurement_name))
                        query.field_type = current_field['type']

                        tags_dict = _get_response(self.hbot.urlfor.dg_measurement_tags(
                            device_id=query.device_name, device_group=query.group_name,
                            table_name=query.measurement_name))
                        query.key_dict = tags_dict

                    if query.group_by_tag_key is  None:
                        query.group_by_tag_key = []
                    else:
                        if not isinstance(query.group_by_tag_key, list):
                            raise ValueError("group_by_tag_key is not a list, please configure a list")

                    # validate tag #TODO
                    if query.group_by_interval is None:
                        query.group_by_interval = "$__interval"
                    if query.group_by_fill is None:
                        query.group_by_fill = "fill(null)"
                    if query.field_aggregation is None:
                        query.field_aggregation = "mean"

            return graphs

        if not isinstance(graphs, list):
            graphs = [graphs]

        ret_val, msg = _validate_data(canvas_name, graphs)
        if not ret_val:
            raise ValueError(msg)

        graphs = _set_default_values(graphs)

        panel_list = []
        graph_type_mapping = {
            "Time Series": "timeseries",
            "Heatmap": "heatmap",
            "Histogram": "histogram"
        }
        unit_value_mapping = {
            "Short": "short",
            "Percentage(0-100)": "percentage",
            "Bits": "bits",
            "Bytes": "bytes",
            "Kilobytes": "kilobytes",
            "Megabytes": "megabytes",
            "Gigabytes": "gigabytes",
            "Parts per million(ppm)": "ppm",
            "Parts per billion(ppb)": "ppb"
        }

        panel_data_list = []
        for graph in graphs:
            panel_data = _get_panel_data(graph)
            panel_data_list = panel_data_list + panel_data
            db = _get_database_name(group_name=graph.query[0].group_name, group_type=graph.query[0].group_type,
                                    device_name=graph.query[0].device_name)
            decimals = graph.decimals
            panel_data = panel_data_list
            panel_name = graph.graph_name
            panel_query = _get_panel_query(panel_data_list)
            panel_type = PanelSchemaPanelType(label=graph.graph_type, value=graph_type_mapping[graph.graph_type])
            panel_uid = _get_panel_uid()
            time_range = PanelSchemaTimeRange(label=graph.time_range, value=graph.time_range)
            unit_type = ""
            if graph.unit_type is not None:
                unit_type = PanelSchemaUnitType(label=graph.unit_type, value=unit_value_mapping[graph.unit_type])

            panels = PanelSchema(db=db, decimals=decimals, panel_data=panel_data, panel_name=panel_name,
                                 panel_query=panel_query, panel_type=panel_type, panel_uid=panel_uid,
                                 time_range=time_range, unit_type=unit_type, y_label=graph.y_label,
                                 y_max=graph.y_max, y_min=graph.y_min)
            panels_value = _get_json(panels.to_dict())
            panel_list.append(panels_value)

        value = {
            "createdAt": "2021-10-11T09:10:30",
            "description": "",
            "name": canvas_name,
            "panels": panel_list,
            "updatedAt": "2021-10-26T04:49:56"
        }
        schema = DatastoreSchema(group_name='grafana', key=canvas_name, value=value)
        return schema

    def get_canvas(self, canvas_name: str = None):
        if canvas_name is not None:
            hb_chart_url = self.hbot.urlfor.data_store_schema(canvas_name)
            # if uncommitted:
            #     hb_chart_url += self.hbot.apiopt_candidate
            response = self.api.get(hb_chart_url)
            if response.status_code != 200:
                logger.error(response.text)
            response.raise_for_status()
            if response.status_code == 404:
                return None
            return response.text
        else:
            print("Please specify the canvas name")

    def add_canvas(self, canvas_name, graphs):
        schema = self.get_add_canvas_schema(canvas_name, graphs)
        payload = self.hbot._create_payload(schema)
        hb_chart_url = self.hbot.urlfor.data_store_schema(canvas_name)
        response = self.api.post(hb_chart_url, json=payload)
        if response.status_code != 200:
            logger.error(response.text)
        response.raise_for_status()
        return True

    def update_canvas(self, canvas_name, graphs):
        schema = self.get_add_canvas_schema(canvas_name, graphs)
        payload = self.hbot._create_payload(schema)
        hb_chart_url = self.hbot.urlfor.data_store_schema(canvas_name)
        response = self.api.put(hb_chart_url, json=payload)
        if response.status_code != 200:
            logger.error(response.text)
        response.raise_for_status()
        return True

    def delete_canvas(self, canvas_name: str = None):
        hb_chart_url = self.hbot.urlfor.data_store_schema(canvas_name)
        response = self.api.delete(hb_chart_url)
        if response.status_code != 204:
            logger.error(response.text)
        response.raise_for_status()

        return True




# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class InstanceScheduleStateSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'group_name': 'str',
        'group_type': 'str',
        'name': 'str',
        'rule': 'str',
        'playbook': 'str',
        'state': 'str',
        'update_time': 'object'
    }

    attribute_map = {
        'group_name': 'group-name',
        'group_type': 'group-type',
        'name': 'name',
        'rule': 'rule',
        'playbook': 'playbook',
        'state': 'state',
        'update_time': 'update-time'
    }

    def __init__(self, group_name=None, group_type=None, name=None, rule=None, playbook=None, state=None, update_time=None):  # noqa: E501
        """InstanceScheduleStateSchema - a model defined in Swagger"""  # noqa: E501

        self._group_name = None
        self._group_type = None
        self._name = None
        self._rule = None
        self._playbook = None
        self._state = None
        self._update_time = None
        self.discriminator = None

        self.group_name = group_name
        self.group_type = group_type
        self.name = name
        self.rule = rule
        self.playbook = playbook
        self.state = state
        if update_time is not None:
            self.update_time = update_time

    @property
    def group_name(self):
        """Gets the group_name of this InstanceScheduleStateSchema.  # noqa: E501

        Name of the group  # noqa: E501

        :return: The group_name of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._group_name

    @group_name.setter
    def group_name(self, group_name):
        """Sets the group_name of this InstanceScheduleStateSchema.

        Name of the group  # noqa: E501

        :param group_name: The group_name of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if group_name is None:
            raise ValueError("Invalid value for `group_name`, must not be `None`")  # noqa: E501

        self._group_name = group_name

    @property
    def group_type(self):
        """Gets the group_type of this InstanceScheduleStateSchema.  # noqa: E501

        Type of the group. Can be one of device-group or network-group  # noqa: E501

        :return: The group_type of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._group_type

    @group_type.setter
    def group_type(self, group_type):
        """Sets the group_type of this InstanceScheduleStateSchema.

        Type of the group. Can be one of device-group or network-group  # noqa: E501

        :param group_type: The group_type of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if group_type is None:
            raise ValueError("Invalid value for `group_type`, must not be `None`")  # noqa: E501
        allowed_values = ["device-group", "network-group"]  # noqa: E501
        if group_type not in allowed_values:
            raise ValueError(
                "Invalid value for `group_type` ({0}), must be one of {1}"  # noqa: E501
                .format(group_type, allowed_values)
            )

        self._group_type = group_type

    @property
    def name(self):
        """Gets the name of this InstanceScheduleStateSchema.  # noqa: E501

        Name of the instance  # noqa: E501

        :return: The name of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this InstanceScheduleStateSchema.

        Name of the instance  # noqa: E501

        :param name: The name of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def rule(self):
        """Gets the rule of this InstanceScheduleStateSchema.  # noqa: E501

        Name of the rule associated with the instance  # noqa: E501

        :return: The rule of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._rule

    @rule.setter
    def rule(self, rule):
        """Sets the rule of this InstanceScheduleStateSchema.

        Name of the rule associated with the instance  # noqa: E501

        :param rule: The rule of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if rule is None:
            raise ValueError("Invalid value for `rule`, must not be `None`")  # noqa: E501

        self._rule = rule

    @property
    def playbook(self):
        """Gets the playbook of this InstanceScheduleStateSchema.  # noqa: E501

        Name of the playbook associated with the instance  # noqa: E501

        :return: The playbook of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._playbook

    @playbook.setter
    def playbook(self, playbook):
        """Sets the playbook of this InstanceScheduleStateSchema.

        Name of the playbook associated with the instance  # noqa: E501

        :param playbook: The playbook of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if playbook is None:
            raise ValueError("Invalid value for `playbook`, must not be `None`")  # noqa: E501

        self._playbook = playbook

    @property
    def state(self):
        """Gets the state of this InstanceScheduleStateSchema.  # noqa: E501

        Scheduled state of the instance. Can be one of active or inactive  # noqa: E501

        :return: The state of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this InstanceScheduleStateSchema.

        Scheduled state of the instance. Can be one of active or inactive  # noqa: E501

        :param state: The state of this InstanceScheduleStateSchema.  # noqa: E501
        :type: str
        """
        if state is None:
            raise ValueError("Invalid value for `state`, must not be `None`")  # noqa: E501
        allowed_values = ["active", "inactive"]  # noqa: E501
        if state not in allowed_values:
            raise ValueError(
                "Invalid value for `state` ({0}), must be one of {1}"  # noqa: E501
                .format(state, allowed_values)
            )

        self._state = state

    @property
    def update_time(self):
        """Gets the update_time of this InstanceScheduleStateSchema.  # noqa: E501

        Time the current status is updated for this instance in elapsed seconds since epoch  # noqa: E501

        :return: The update_time of this InstanceScheduleStateSchema.  # noqa: E501
        :rtype: object
        """
        return self._update_time

    @update_time.setter
    def update_time(self, update_time):
        """Sets the update_time of this InstanceScheduleStateSchema.

        Time the current status is updated for this instance in elapsed seconds since epoch  # noqa: E501

        :param update_time: The update_time of this InstanceScheduleStateSchema.  # noqa: E501
        :type: object
        """

        self._update_time = update_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InstanceScheduleStateSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InstanceScheduleStateSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class DeviceSchemaVendorCisco(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'operating_system': 'str',
        'platform': 'str',
        'product': 'str',
        'release': 'str'
    }

    attribute_map = {
        'operating_system': 'operating-system',
        'platform': 'platform',
        'product': 'product',
        'release': 'release'
    }

    def __init__(self, operating_system=None, platform=None, product=None, release=None):  # noqa: E501
        """DeviceSchemaVendorCisco - a model defined in Swagger"""  # noqa: E501

        self._operating_system = None
        self._platform = None
        self._product = None
        self._release = None
        self.discriminator = None

        self.operating_system = operating_system
        if platform is not None:
            self.platform = platform
        if product is not None:
            self.product = product
        if release is not None:
            self.release = release

    @property
    def operating_system(self):
        """Gets the operating_system of this DeviceSchemaVendorCisco.  # noqa: E501

        Operating system of the device  # noqa: E501

        :return: The operating_system of this DeviceSchemaVendorCisco.  # noqa: E501
        :rtype: str
        """
        return self._operating_system

    @operating_system.setter
    def operating_system(self, operating_system):
        """Sets the operating_system of this DeviceSchemaVendorCisco.

        Operating system of the device  # noqa: E501

        :param operating_system: The operating_system of this DeviceSchemaVendorCisco.  # noqa: E501
        :type: str
        """
        if operating_system is None:
            raise ValueError("Invalid value for `operating_system`, must not be `None`")  # noqa: E501
        allowed_values = ["iosxr", "nxos"]  # noqa: E501
        if operating_system not in allowed_values:
            raise ValueError(
                "Invalid value for `operating_system` ({0}), must be one of {1}"  # noqa: E501
                .format(operating_system, allowed_values)
            )

        self._operating_system = operating_system

    @property
    def platform(self):
        """Gets the platform of this DeviceSchemaVendorCisco.  # noqa: E501

        Platform name of the device, Example: MX240  # noqa: E501

        :return: The platform of this DeviceSchemaVendorCisco.  # noqa: E501
        :rtype: str
        """
        return self._platform

    @platform.setter
    def platform(self, platform):
        """Sets the platform of this DeviceSchemaVendorCisco.

        Platform name of the device, Example: MX240  # noqa: E501

        :param platform: The platform of this DeviceSchemaVendorCisco.  # noqa: E501
        :type: str
        """
        if platform is not None and len(platform) > 64:
            raise ValueError("Invalid value for `platform`, length must be less than or equal to `64`")  # noqa: E501
        if platform is not None and len(platform) < 1:
            raise ValueError("Invalid value for `platform`, length must be greater than or equal to `1`")  # noqa: E501

        self._platform = platform

    @property
    def product(self):
        """Gets the product of this DeviceSchemaVendorCisco.  # noqa: E501

        Product category of the device, Example: MX  # noqa: E501

        :return: The product of this DeviceSchemaVendorCisco.  # noqa: E501
        :rtype: str
        """
        return self._product

    @product.setter
    def product(self, product):
        """Sets the product of this DeviceSchemaVendorCisco.

        Product category of the device, Example: MX  # noqa: E501

        :param product: The product of this DeviceSchemaVendorCisco.  # noqa: E501
        :type: str
        """
        if product is not None and len(product) > 64:
            raise ValueError("Invalid value for `product`, length must be less than or equal to `64`")  # noqa: E501
        if product is not None and len(product) < 1:
            raise ValueError("Invalid value for `product`, length must be greater than or equal to `1`")  # noqa: E501

        self._product = product

    @property
    def release(self):
        """Gets the release of this DeviceSchemaVendorCisco.  # noqa: E501

        Release string of the device, Example: 19.2R1  # noqa: E501

        :return: The release of this DeviceSchemaVendorCisco.  # noqa: E501
        :rtype: str
        """
        return self._release

    @release.setter
    def release(self, release):
        """Sets the release of this DeviceSchemaVendorCisco.

        Release string of the device, Example: 19.2R1  # noqa: E501

        :param release: The release of this DeviceSchemaVendorCisco.  # noqa: E501
        :type: str
        """
        if release is not None and len(release) > 64:
            raise ValueError("Invalid value for `release`, length must be less than or equal to `64`")  # noqa: E501
        if release is not None and len(release) < 1:
            raise ValueError("Invalid value for `release`, length must be greater than or equal to `1`")  # noqa: E501

        self._release = release

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DeviceSchemaVendorCisco, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DeviceSchemaVendorCisco):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class ProfileSchemaDatasummarizationPath(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'aggregation_functions': 'list[str]',
        'name': 'str'
    }

    attribute_map = {
        'aggregation_functions': 'aggregation-functions',
        'name': 'name'
    }

    def __init__(self, aggregation_functions=None, name=None):  # noqa: E501
        """ProfileSchemaDatasummarizationPath - a model defined in Swagger"""  # noqa: E501

        self._aggregation_functions = None
        self._name = None
        self.discriminator = None

        self.aggregation_functions = aggregation_functions
        self.name = name

    @property
    def aggregation_functions(self):
        """Gets the aggregation_functions of this ProfileSchemaDatasummarizationPath.  # noqa: E501


        :return: The aggregation_functions of this ProfileSchemaDatasummarizationPath.  # noqa: E501
        :rtype: list[str]
        """
        return self._aggregation_functions

    @aggregation_functions.setter
    def aggregation_functions(self, aggregation_functions):
        """Sets the aggregation_functions of this ProfileSchemaDatasummarizationPath.


        :param aggregation_functions: The aggregation_functions of this ProfileSchemaDatasummarizationPath.  # noqa: E501
        :type: list[str]
        """
        if aggregation_functions is None:
            raise ValueError("Invalid value for `aggregation_functions`, must not be `None`")  # noqa: E501
        allowed_values = ["latest", "sum", "count", "mean", "min", "max", "on-change", "stddev"]  # noqa: E501
        if not set(aggregation_functions).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `aggregation_functions` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(aggregation_functions) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._aggregation_functions = aggregation_functions

    @property
    def name(self):
        """Gets the name of this ProfileSchemaDatasummarizationPath.  # noqa: E501

        Sensor field path for which summarization should be changed. Apart from JTI OC sensor path, '<sensor-name>:' should be prepended to the sensor path  # noqa: E501

        :return: The name of this ProfileSchemaDatasummarizationPath.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ProfileSchemaDatasummarizationPath.

        Sensor field path for which summarization should be changed. Apart from JTI OC sensor path, '<sensor-name>:' should be prepended to the sensor path  # noqa: E501

        :param name: The name of this ProfileSchemaDatasummarizationPath.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProfileSchemaDatasummarizationPath, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProfileSchemaDatasummarizationPath):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Healthbot APIs

    API interface for Healthbot application  # noqa: E501

    OpenAPI spec version: 3.1.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class InlineResponse2004(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'user_id': 'str',
        'user_name': 'str',
        'first_name': 'str',
        'last_name': 'str',
        'email': 'str',
        'active': 'bool',
        'groups': 'AssociatedGroupSchema'
    }

    attribute_map = {
        'user_id': 'userId',
        'user_name': 'userName',
        'first_name': 'firstName',
        'last_name': 'lastName',
        'email': 'email',
        'active': 'active',
        'groups': 'groups'
    }

    def __init__(self, user_id=None, user_name=None, first_name=None, last_name=None, email=None, active=None, groups=None):  # noqa: E501
        """InlineResponse2004 - a model defined in Swagger"""  # noqa: E501

        self._user_id = None
        self._user_name = None
        self._first_name = None
        self._last_name = None
        self._email = None
        self._active = None
        self._groups = None
        self.discriminator = None

        if user_id is not None:
            self.user_id = user_id
        if user_name is not None:
            self.user_name = user_name
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if email is not None:
            self.email = email
        if active is not None:
            self.active = active
        if groups is not None:
            self.groups = groups

    @property
    def user_id(self):
        """Gets the user_id of this InlineResponse2004.  # noqa: E501

        ID generated by system  # noqa: E501

        :return: The user_id of this InlineResponse2004.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this InlineResponse2004.

        ID generated by system  # noqa: E501

        :param user_id: The user_id of this InlineResponse2004.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def user_name(self):
        """Gets the user_name of this InlineResponse2004.  # noqa: E501

        Name of the user  # noqa: E501

        :return: The user_name of this InlineResponse2004.  # noqa: E501
        :rtype: str
        """
        return self._user_name

    @user_name.setter
    def user_name(self, user_name):
        """Sets the user_name of this InlineResponse2004.

        Name of the user  # noqa: E501

        :param user_name: The user_name of this InlineResponse2004.  # noqa: E501
        :type: str
        """

        self._user_name = user_name

    @property
    def first_name(self):
        """Gets the first_name of this InlineResponse2004.  # noqa: E501

        First name of the user  # noqa: E501

        :return: The first_name of this InlineResponse2004.  # noqa: E501
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """Sets the first_name of this InlineResponse2004.

        First name of the user  # noqa: E501

        :param first_name: The first_name of this InlineResponse2004.  # noqa: E501
        :type: str
        """

        self._first_name = first_name

    @property
    def last_name(self):
        """Gets the last_name of this InlineResponse2004.  # noqa: E501

        Last name of the user  # noqa: E501

        :return: The last_name of this InlineResponse2004.  # noqa: E501
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """Sets the last_name of this InlineResponse2004.

        Last name of the user  # noqa: E501

        :param last_name: The last_name of this InlineResponse2004.  # noqa: E501
        :type: str
        """

        self._last_name = last_name

    @property
    def email(self):
        """Gets the email of this InlineResponse2004.  # noqa: E501

        Email of the user  # noqa: E501

        :return: The email of this InlineResponse2004.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this InlineResponse2004.

        Email of the user  # noqa: E501

        :param email: The email of this InlineResponse2004.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def active(self):
        """Gets the active of this InlineResponse2004.  # noqa: E501

        Status of the user  # noqa: E501

        :return: The active of this InlineResponse2004.  # noqa: E501
        :rtype: bool
        """
        return self._active

    @active.setter
    def active(self, active):
        """Sets the active of this InlineResponse2004.

        Status of the user  # noqa: E501

        :param active: The active of this InlineResponse2004.  # noqa: E501
        :type: bool
        """

        self._active = active

    @property
    def groups(self):
        """Gets the groups of this InlineResponse2004.  # noqa: E501


        :return: The groups of this InlineResponse2004.  # noqa: E501
        :rtype: AssociatedGroupSchema
        """
        return self._groups

    @groups.setter
    def groups(self, groups):
        """Sets the groups of this InlineResponse2004.


        :param groups: The groups of this InlineResponse2004.  # noqa: E501
        :type: AssociatedGroupSchema
        """

        self._groups = groups

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InlineResponse2004, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InlineResponse2004):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class HbGraphsQuery(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'group_name': 'str',
        'group_type': 'str',
        'device_name': 'str',
        'measurement_name': 'str',
        'measurement_type': 'str',
        'transformation': 'str',
        'field_name': 'str',
        'field_type': 'str',
        'field_aggregation': 'str',
        'where': 'HbGraphsQueryWhere',
        'group_by_interval': 'str',
        'group_by_fill': 'str',
        'group_by_tag_key': 'str',
        'retention_policy': 'str'
    }

    attribute_map = {
        'group_name': 'group_name',
        'group_type': 'group_type',
        'device_name': 'device_name',
        'measurement_name': 'measurement_name',
        'measurement_type': 'measurement_type',
        'transformation': 'transformation',
        'field_name': 'field_name',
        'field_type': 'field_type',
        'field_aggregation': 'field_aggregation',
        'where': 'where',
        'group_by_interval': 'group_by_interval',
        'group_by_fill': 'group_by_fill',
        'group_by_tag_key': 'group_by_tag_key',
        'retention_policy': 'retention_policy'
    }

    def __init__(self, group_name=None, group_type=None, device_name=None, measurement_name=None, measurement_type=None, transformation=None, field_name=None, field_type=None, field_aggregation=None, where=None, group_by_interval=None, group_by_fill=None, group_by_tag_key=None, retention_policy=None):  # noqa: E501
        """HbGraphsQuery - a model defined in Swagger"""  # noqa: E501

        self._group_name = None
        self._group_type = None
        self._device_name = None
        self._measurement_name = None
        self._measurement_type = None
        self._transformation = None
        self._field_name = None
        self._field_type = None
        self._field_aggregation = None
        self._where = None
        self._group_by_interval = None
        self._group_by_fill = None
        self._group_by_tag_key = None
        self._retention_policy = None
        self.discriminator = None

        if group_name is not None:
            self.group_name = group_name
        if group_type is not None:
            self.group_type = group_type
        if device_name is not None:
            self.device_name = device_name
        if measurement_name is not None:
            self.measurement_name = measurement_name
        if measurement_type is not None:
            self.measurement_type = measurement_type
        if transformation is not None:
            self.transformation = transformation
        if field_name is not None:
            self.field_name = field_name
        if field_type is not None:
            self.field_type = field_type
        if field_aggregation is not None:
            self.field_aggregation = field_aggregation
        if where is not None:
            self.where = where
        if group_by_interval is not None:
            self.group_by_interval = group_by_interval
        if group_by_fill is not None:
            self.group_by_fill = group_by_fill
        if group_by_tag_key is not None:
            self.group_by_tag_key = group_by_tag_key
        if retention_policy is not None:
            self.retention_policy = retention_policy

    @property
    def group_name(self):
        """Gets the group_name of this HbGraphsQuery.  # noqa: E501

        Device/Network group name  # noqa: E501

        :return: The group_name of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._group_name

    @group_name.setter
    def group_name(self, group_name):
        """Sets the group_name of this HbGraphsQuery.

        Device/Network group name  # noqa: E501

        :param group_name: The group_name of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._group_name = group_name

    @property
    def group_type(self):
        """Gets the group_type of this HbGraphsQuery.  # noqa: E501

        Device/Network group type  # noqa: E501

        :return: The group_type of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._group_type

    @group_type.setter
    def group_type(self, group_type):
        """Sets the group_type of this HbGraphsQuery.

        Device/Network group type  # noqa: E501

        :param group_type: The group_type of this HbGraphsQuery.  # noqa: E501
        :type: str
        """
        allowed_values = ["device", "network"]  # noqa: E501
        if group_type not in allowed_values:
            raise ValueError(
                "Invalid value for `group_type` ({0}), must be one of {1}"  # noqa: E501
                .format(group_type, allowed_values)
            )

        self._group_type = group_type

    @property
    def device_name(self):
        """Gets the device_name of this HbGraphsQuery.  # noqa: E501

        label name  # noqa: E501

        :return: The device_name of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._device_name

    @device_name.setter
    def device_name(self, device_name):
        """Sets the device_name of this HbGraphsQuery.

        label name  # noqa: E501

        :param device_name: The device_name of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._device_name = device_name

    @property
    def measurement_name(self):
        """Gets the measurement_name of this HbGraphsQuery.  # noqa: E501

        Measurement name (topic/rule name)  # noqa: E501

        :return: The measurement_name of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._measurement_name

    @measurement_name.setter
    def measurement_name(self, measurement_name):
        """Sets the measurement_name of this HbGraphsQuery.

        Measurement name (topic/rule name)  # noqa: E501

        :param measurement_name: The measurement_name of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._measurement_name = measurement_name

    @property
    def measurement_type(self):
        """Gets the measurement_type of this HbGraphsQuery.  # noqa: E501

        Measurement type: Field table/Trigger table/Rollup table  # noqa: E501

        :return: The measurement_type of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._measurement_type

    @measurement_type.setter
    def measurement_type(self, measurement_type):
        """Sets the measurement_type of this HbGraphsQuery.

        Measurement type: Field table/Trigger table/Rollup table  # noqa: E501

        :param measurement_type: The measurement_type of this HbGraphsQuery.  # noqa: E501
        :type: str
        """
        allowed_values = ["Field table", "Trigger table", "Rollup table"]  # noqa: E501
        if measurement_type not in allowed_values:
            raise ValueError(
                "Invalid value for `measurement_type` ({0}), must be one of {1}"  # noqa: E501
                .format(measurement_type, allowed_values)
            )

        self._measurement_type = measurement_type

    @property
    def transformation(self):
        """Gets the transformation of this HbGraphsQuery.  # noqa: E501

        Transformation value for query  # noqa: E501

        :return: The transformation of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._transformation

    @transformation.setter
    def transformation(self, transformation):
        """Sets the transformation of this HbGraphsQuery.

        Transformation value for query  # noqa: E501

        :param transformation: The transformation of this HbGraphsQuery.  # noqa: E501
        :type: str
        """
        allowed_values = ["derivative", "spread", "non-negative-derivative", "difference", "cumulative-sum", "elapsed"]  # noqa: E501
        if transformation not in allowed_values:
            raise ValueError(
                "Invalid value for `transformation` ({0}), must be one of {1}"  # noqa: E501
                .format(transformation, allowed_values)
            )

        self._transformation = transformation

    @property
    def field_name(self):
        """Gets the field_name of this HbGraphsQuery.  # noqa: E501

        Field name of a measurement   # noqa: E501

        :return: The field_name of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._field_name

    @field_name.setter
    def field_name(self, field_name):
        """Sets the field_name of this HbGraphsQuery.

        Field name of a measurement   # noqa: E501

        :param field_name: The field_name of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._field_name = field_name

    @property
    def field_type(self):
        """Gets the field_type of this HbGraphsQuery.  # noqa: E501

        Field type of the measurement (int, float, string, uint)  # noqa: E501

        :return: The field_type of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._field_type

    @field_type.setter
    def field_type(self, field_type):
        """Sets the field_type of this HbGraphsQuery.

        Field type of the measurement (int, float, string, uint)  # noqa: E501

        :param field_type: The field_type of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._field_type = field_type

    @property
    def field_aggregation(self):
        """Gets the field_aggregation of this HbGraphsQuery.  # noqa: E501

        Data aggregation type of the field/key  # noqa: E501

        :return: The field_aggregation of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._field_aggregation

    @field_aggregation.setter
    def field_aggregation(self, field_aggregation):
        """Sets the field_aggregation of this HbGraphsQuery.

        Data aggregation type of the field/key  # noqa: E501

        :param field_aggregation: The field_aggregation of this HbGraphsQuery.  # noqa: E501
        :type: str
        """
        allowed_values = ["mean", "mode", "median", "count", "sum", "integral", "distinct"]  # noqa: E501
        if field_aggregation not in allowed_values:
            raise ValueError(
                "Invalid value for `field_aggregation` ({0}), must be one of {1}"  # noqa: E501
                .format(field_aggregation, allowed_values)
            )

        self._field_aggregation = field_aggregation

    @property
    def where(self):
        """Gets the where of this HbGraphsQuery.  # noqa: E501


        :return: The where of this HbGraphsQuery.  # noqa: E501
        :rtype: HbGraphsQueryWhere
        """
        return self._where

    @where.setter
    def where(self, where):
        """Sets the where of this HbGraphsQuery.


        :param where: The where of this HbGraphsQuery.  # noqa: E501
        :type: HbGraphsQueryWhere
        """

        self._where = where

    @property
    def group_by_interval(self):
        """Gets the group_by_interval of this HbGraphsQuery.  # noqa: E501

        Group by interval of the query  # noqa: E501

        :return: The group_by_interval of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._group_by_interval

    @group_by_interval.setter
    def group_by_interval(self, group_by_interval):
        """Sets the group_by_interval of this HbGraphsQuery.

        Group by interval of the query  # noqa: E501

        :param group_by_interval: The group_by_interval of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._group_by_interval = group_by_interval

    @property
    def group_by_fill(self):
        """Gets the group_by_fill of this HbGraphsQuery.  # noqa: E501

        Group by fill value of the query  # noqa: E501

        :return: The group_by_fill of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._group_by_fill

    @group_by_fill.setter
    def group_by_fill(self, group_by_fill):
        """Sets the group_by_fill of this HbGraphsQuery.

        Group by fill value of the query  # noqa: E501

        :param group_by_fill: The group_by_fill of this HbGraphsQuery.  # noqa: E501
        :type: str
        """
        allowed_values = ["fill(null)", "none"]  # noqa: E501
        if group_by_fill not in allowed_values:
            raise ValueError(
                "Invalid value for `group_by_fill` ({0}), must be one of {1}"  # noqa: E501
                .format(group_by_fill, allowed_values)
            )

        self._group_by_fill = group_by_fill

    @property
    def group_by_tag_key(self):
        """Gets the group_by_tag_key of this HbGraphsQuery.  # noqa: E501

        Group by tag key value of the query  # noqa: E501

        :return: The group_by_tag_key of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._group_by_tag_key

    @group_by_tag_key.setter
    def group_by_tag_key(self, group_by_tag_key):
        """Sets the group_by_tag_key of this HbGraphsQuery.

        Group by tag key value of the query  # noqa: E501

        :param group_by_tag_key: The group_by_tag_key of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._group_by_tag_key = group_by_tag_key

    @property
    def retention_policy(self):
        """Gets the retention_policy of this HbGraphsQuery.  # noqa: E501

        Retention policy name  # noqa: E501

        :return: The retention_policy of this HbGraphsQuery.  # noqa: E501
        :rtype: str
        """
        return self._retention_policy

    @retention_policy.setter
    def retention_policy(self, retention_policy):
        """Sets the retention_policy of this HbGraphsQuery.

        Retention policy name  # noqa: E501

        :param retention_policy: The retention_policy of this HbGraphsQuery.  # noqa: E501
        :type: str
        """

        self._retention_policy = retention_policy

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(HbGraphsQuery, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, HbGraphsQuery):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class RuleSchemaFormula1(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        '_and': 'RuleSchemaFormula1And',
        '_or': 'RuleSchemaFormula1Or',
        'unique': 'RuleSchemaFormula1Unique',
        'unless': 'RuleSchemaFormula1Unless'
    }

    attribute_map = {
        '_and': 'and',
        '_or': 'or',
        'unique': 'unique',
        'unless': 'unless'
    }

    def __init__(self, _and=None, _or=None, unique=None, unless=None):  # noqa: E501
        """RuleSchemaFormula1 - a model defined in Swagger"""  # noqa: E501

        self.__and = None
        self.__or = None
        self._unique = None
        self._unless = None
        self.discriminator = None

        if _and is not None:
            self._and = _and
        if _or is not None:
            self._or = _or
        if unique is not None:
            self.unique = unique
        if unless is not None:
            self.unless = unless

    @property
    def _and(self):
        """Gets the _and of this RuleSchemaFormula1.  # noqa: E501


        :return: The _and of this RuleSchemaFormula1.  # noqa: E501
        :rtype: RuleSchemaFormula1And
        """
        return self.__and

    @_and.setter
    def _and(self, _and):
        """Sets the _and of this RuleSchemaFormula1.


        :param _and: The _and of this RuleSchemaFormula1.  # noqa: E501
        :type: RuleSchemaFormula1And
        """

        self.__and = _and

    @property
    def _or(self):
        """Gets the _or of this RuleSchemaFormula1.  # noqa: E501


        :return: The _or of this RuleSchemaFormula1.  # noqa: E501
        :rtype: RuleSchemaFormula1Or
        """
        return self.__or

    @_or.setter
    def _or(self, _or):
        """Sets the _or of this RuleSchemaFormula1.


        :param _or: The _or of this RuleSchemaFormula1.  # noqa: E501
        :type: RuleSchemaFormula1Or
        """

        self.__or = _or

    @property
    def unique(self):
        """Gets the unique of this RuleSchemaFormula1.  # noqa: E501


        :return: The unique of this RuleSchemaFormula1.  # noqa: E501
        :rtype: RuleSchemaFormula1Unique
        """
        return self._unique

    @unique.setter
    def unique(self, unique):
        """Sets the unique of this RuleSchemaFormula1.


        :param unique: The unique of this RuleSchemaFormula1.  # noqa: E501
        :type: RuleSchemaFormula1Unique
        """

        self._unique = unique

    @property
    def unless(self):
        """Gets the unless of this RuleSchemaFormula1.  # noqa: E501


        :return: The unless of this RuleSchemaFormula1.  # noqa: E501
        :rtype: RuleSchemaFormula1Unless
        """
        return self._unless

    @unless.setter
    def unless(self, unless):
        """Sets the unless of this RuleSchemaFormula1.


        :param unless: The unless of this RuleSchemaFormula1.  # noqa: E501
        :type: RuleSchemaFormula1Unless
        """

        self._unless = unless

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuleSchemaFormula1, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuleSchemaFormula1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TsdbPostBodyItemsDeviceAggregation(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'add_device_as_tag': 'bool',
        'bottom_limit': 'int',
        'field': 'str',
        'top_limit': 'int'
    }

    attribute_map = {
        'add_device_as_tag': 'addDeviceAsTag',
        'bottom_limit': 'bottomLimit',
        'field': 'field',
        'top_limit': 'topLimit'
    }

    def __init__(self, add_device_as_tag=None, bottom_limit=None, field=None, top_limit=None):  # noqa: E501
        """TsdbPostBodyItemsDeviceAggregation - a model defined in Swagger"""  # noqa: E501

        self._add_device_as_tag = None
        self._bottom_limit = None
        self._field = None
        self._top_limit = None
        self.discriminator = None

        if add_device_as_tag is not None:
            self.add_device_as_tag = add_device_as_tag
        if bottom_limit is not None:
            self.bottom_limit = bottom_limit
        self.field = field
        if top_limit is not None:
            self.top_limit = top_limit

    @property
    def add_device_as_tag(self):
        """Gets the add_device_as_tag of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501

        Add device-id tag as basis for aggregation  # noqa: E501

        :return: The add_device_as_tag of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :rtype: bool
        """
        return self._add_device_as_tag

    @add_device_as_tag.setter
    def add_device_as_tag(self, add_device_as_tag):
        """Sets the add_device_as_tag of this TsdbPostBodyItemsDeviceAggregation.

        Add device-id tag as basis for aggregation  # noqa: E501

        :param add_device_as_tag: The add_device_as_tag of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :type: bool
        """

        self._add_device_as_tag = add_device_as_tag

    @property
    def bottom_limit(self):
        """Gets the bottom_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501

        Fetch bottom N results  # noqa: E501

        :return: The bottom_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :rtype: int
        """
        return self._bottom_limit

    @bottom_limit.setter
    def bottom_limit(self, bottom_limit):
        """Sets the bottom_limit of this TsdbPostBodyItemsDeviceAggregation.

        Fetch bottom N results  # noqa: E501

        :param bottom_limit: The bottom_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :type: int
        """

        self._bottom_limit = bottom_limit

    @property
    def field(self):
        """Gets the field of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501

        Field based on whiuch to aggregate data  # noqa: E501

        :return: The field of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :rtype: str
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this TsdbPostBodyItemsDeviceAggregation.

        Field based on whiuch to aggregate data  # noqa: E501

        :param field: The field of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :type: str
        """
        if field is None:
            raise ValueError("Invalid value for `field`, must not be `None`")  # noqa: E501

        self._field = field

    @property
    def top_limit(self):
        """Gets the top_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501

        Fetch Top N results  # noqa: E501

        :return: The top_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :rtype: int
        """
        return self._top_limit

    @top_limit.setter
    def top_limit(self, top_limit):
        """Sets the top_limit of this TsdbPostBodyItemsDeviceAggregation.

        Fetch Top N results  # noqa: E501

        :param top_limit: The top_limit of this TsdbPostBodyItemsDeviceAggregation.  # noqa: E501
        :type: int
        """

        self._top_limit = top_limit

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TsdbPostBodyItemsDeviceAggregation, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TsdbPostBodyItemsDeviceAggregation):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
